@@ -12,8 +12,7 @@ import (
 	"sort"
 	"strings"
 
-	"github.com/gruntwork-io/terragrunt/internal/runner"
-	"github.com/gruntwork-io/terragrunt/internal/runner/common"
+	"github.com/gruntwork-io/terragrunt/internal/discovery"
 
 	"github.com/google/shlex"
 	"github.com/hashicorp/hcl/v2"
@@ -36,11 +35,6 @@ import (
 
 const splitCount = 2
 
-// diagnostics to skip because they are not relevant for terragrunt validation
-var skipHCLDiagnostics = []string{
-	"there is no variable named \"dependency\"",
-}
-
 func Run(ctx context.Context, l log.Logger, opts *options.TerragruntOptions) error {
 	if opts.HCLValidateInputs {
 		if opts.HCLValidateShowConfigPath {
@@ -67,24 +61,6 @@ func RunValidate(ctx context.Context, l log.Logger, opts *options.TerragruntOpti
 	parseOptions := []hclparse.Option{
 		hclparse.WithDiagnosticsHandler(func(file *hcl.File, hclDiags hcl.Diagnostics) (hcl.Diagnostics, error) {
 			for _, hclDiag := range hclDiags {
-				// Only apply skip list when using --show-config-path flag
-				// This flag is used by the deprecated hclvalidate command to show only file paths
-				if opts.HCLValidateShowConfigPath {
-					skip := false
-
-					for _, skipMsg := range skipHCLDiagnostics {
-						// check if the diagnostic message contains any of the skip messages in lower case
-						if strings.Contains(strings.ToLower(hclDiag.Detail), strings.ToLower(skipMsg)) {
-							skip = true
-							break
-						}
-					}
-
-					if skip {
-						continue
-					}
-				}
-
 				// Only report diagnostics that are actually in the file being parsed,
 				// not errors from dependencies or other files
 				if hclDiag.Subject != nil && file != nil {
@@ -108,25 +84,40 @@ func RunValidate(ctx context.Context, l log.Logger, opts *options.TerragruntOpti
 
 	opts.SkipOutput = true
 	opts.NonInteractive = true
-	opts.RunTerragrunt = func(ctx context.Context, l log.Logger, opts *options.TerragruntOptions, r *report.Report) error {
-		_, err := config.ReadTerragruntConfig(ctx, l, opts, parseOptions)
-		return err
-	}
 
-	stack, err := runner.FindStackInSubfolders(
-		ctx,
-		l,
-		opts,
-		common.WithParseOptions(parseOptions),
-		common.WithReport(report.NewReport()),
-	)
+	// Discover Terragrunt configurations and validate by parsing them directly
+	d := discovery.NewDiscovery(opts.WorkingDir).WithParserOptions(parseOptions)
+
+	components, err := d.Discover(ctx, l, opts)
 	if err != nil {
 		return processDiagnostics(l, opts, diags, err)
 	}
 
-	stackErr := stack.Run(ctx, l, opts)
+	parseErrs := []error{}
 
-	return processDiagnostics(l, opts, diags, stackErr)
+	for _, c := range components {
+		parseOpts := opts.Clone()
+		parseOpts.WorkingDir = c.Path
+
+		// Determine which config filename to use for a full parse
+		configFilename := config.DefaultTerragruntConfigPath
+		if len(opts.TerragruntConfigPath) > 0 {
+			configFilename = filepath.Base(opts.TerragruntConfigPath)
+		}
+
+		parseOpts.TerragruntConfigPath = filepath.Join(c.Path, configFilename)
+
+		if _, err := config.ReadTerragruntConfig(ctx, l, parseOpts, parseOptions); err != nil {
+			parseErrs = append(parseErrs, errors.New(err))
+		}
+	}
+
+	var combinedErr error
+	if len(parseErrs) > 0 {
+		combinedErr = errors.Join(parseErrs...)
+	}
+
+	return processDiagnostics(l, opts, diags, combinedErr)
 }
 
 func processDiagnostics(l log.Logger, opts *options.TerragruntOptions, diags diagnostic.Diagnostics, callErr error) error {
