@@ -1,8 +1,10 @@
 package discovery
 
 import (
+	"github.com/gruntwork-io/terragrunt/internal/component"
 	"github.com/gruntwork-io/terragrunt/internal/experiment"
 	"github.com/gruntwork-io/terragrunt/internal/filter"
+	"github.com/mattn/go-shellwords"
 )
 
 // DiscoveryCommandOptions contains options for discovery commands like find and list.
@@ -16,6 +18,7 @@ type DiscoveryCommandOptions struct {
 	External         bool
 	Exclude          bool
 	Include          bool
+	Reading          bool
 }
 
 // NewForCommand creates a Discovery configured for discovery commands (find/list).
@@ -45,6 +48,34 @@ func NewForCommand(opts DiscoveryCommandOptions) (*Discovery, error) {
 		d = d.WithParseInclude()
 	}
 
+	if opts.Reading {
+		d = d.WithReadFiles()
+	}
+
+	if opts.QueueConstructAs != "" {
+		d = d.WithParseExclude()
+		d = d.WithDiscoverDependencies()
+
+		parser := shellwords.NewParser()
+
+		args, err := parser.Parse(opts.QueueConstructAs)
+		if err != nil {
+			return nil, err
+		}
+
+		cmd := args[0]
+		if len(args) > 1 {
+			args = args[1:]
+		} else {
+			args = nil
+		}
+
+		d = d.WithDiscoveryContext(&component.DiscoveryContext{
+			Cmd:  cmd,
+			Args: args,
+		})
+	}
+
 	if opts.Experiments.Evaluate(experiment.FilterFlag) && len(opts.FilterQueries) > 0 {
 		filters, err := filter.ParseFilterQueries(opts.FilterQueries, opts.WorkingDir)
 		if err != nil {
