@@ -89,6 +89,12 @@ type Discovery struct {
 	// sort determines the sort order of the discovered configurations.
 	sort Sort
 
+	// compiledIncludePatterns are precompiled glob patterns for includeDirs.
+	compiledIncludePatterns []CompiledPattern
+
+	// compiledExcludePatterns are precompiled glob patterns for excludeDirs.
+	compiledExcludePatterns []CompiledPattern
+
 	// configFilenames is the list of config filenames to discover. If nil, defaults are used.
 	configFilenames []string
 
@@ -98,12 +104,6 @@ type Discovery struct {
 	// excludeDirs is a list of directory patterns to exclude from discovery.
 	excludeDirs []string
 
-	// compiledIncludePatterns are precompiled glob patterns for includeDirs.
-	compiledIncludePatterns []CompiledPattern
-
-	// compiledExcludePatterns are precompiled glob patterns for excludeDirs.
-	compiledExcludePatterns []CompiledPattern
-
 	// parserOptions are custom HCL parser options to use when parsing during discovery
 	parserOptions []hclparse.Option
 
@@ -113,8 +113,14 @@ type Discovery struct {
 	// hiddenDirMemo is a memoization of hidden directories.
 	hiddenDirMemo hiddenDirMemo
 
-	// strictInclude determines whether to use strict include mode (only include directories that match includeDirs).
-	strictInclude bool
+	// numWorkers determines the number of concurrent workers for discovery operations.
+	numWorkers int
+
+	// maxDependencyDepth is the maximum depth of the dependency tree to discover.
+	maxDependencyDepth int
+
+	// discoverDependencies determines whether to discover dependencies.
+	discoverDependencies bool
 
 	// excludeByDefault determines whether to exclude configurations by default (triggered by include flags).
 	excludeByDefault bool
@@ -122,20 +128,14 @@ type Discovery struct {
 	// ignoreExternalDependencies determines whether to drop dependencies that are outside the working directory.
 	ignoreExternalDependencies bool
 
-	// maxDependencyDepth is the maximum depth of the dependency tree to discover.
-	maxDependencyDepth int
-
-	// numWorkers determines the number of concurrent workers for discovery operations.
-	numWorkers int
-
 	// hidden determines whether to detect configurations in hidden directories.
 	hidden bool
 
 	// requiresParse is true when the discovery requires parsing Terragrunt configurations.
 	requiresParse bool
 
-	// discoverDependencies determines whether to discover dependencies.
-	discoverDependencies bool
+	// strictInclude determines whether to use strict include mode (only include directories that match includeDirs).
+	strictInclude bool
 
 	// parseExclude determines whether to parse exclude configurations.
 	parseExclude bool
@@ -143,6 +143,9 @@ type Discovery struct {
 	// parseInclude determines whether to parse include configurations.
 	parseInclude bool
 
+	// readFiles determines whether to parse for reading files.
+	readFiles bool
+
 	// discoverExternalDependencies determines whether to discover external dependencies.
 	discoverExternalDependencies bool
 
@@ -203,9 +206,9 @@ func (d *Discovery) WithSort(sort Sort) *Discovery {
 
 // WithDiscoverDependencies sets the DiscoverDependencies flag to true.
 func (d *Discovery) WithDiscoverDependencies() *Discovery {
-	d.discoverDependencies = true
+	d.WithRequiresParse()
 
-	d.requiresParse = true
+	d.discoverDependencies = true
 
 	if d.maxDependencyDepth == 0 {
 		d.maxDependencyDepth = defaultMaxDependencyDepth
@@ -214,19 +217,35 @@ func (d *Discovery) WithDiscoverDependencies() *Discovery {
 	return d
 }
 
-// WithParseExclude sets the ParseExclude flag to true.
+// WithParseExclude sets the parseExclude flag to true.
 func (d *Discovery) WithParseExclude() *Discovery {
-	d.parseExclude = true
+	d.WithRequiresParse()
 
-	d.requiresParse = true
+	d.parseExclude = true
 
 	return d
 }
 
-// WithParseInclude sets the ParseExclude flag to true.
+// WithParseInclude sets the parseInclude flag to true.
 func (d *Discovery) WithParseInclude() *Discovery {
+	d.WithRequiresParse()
+
 	d.parseInclude = true
 
+	return d
+}
+
+// WithReadFiles sets the readFiles flag to true.
+func (d *Discovery) WithReadFiles() *Discovery {
+	d.WithRequiresParse()
+
+	d.readFiles = true
+
+	return d
+}
+
+// WithRequiresParse sets the requiresParse flag to true.
+func (d *Discovery) WithRequiresParse() *Discovery {
 	d.requiresParse = true
 
 	return d
@@ -387,7 +406,7 @@ func String(c *component.Component) string {
 // ContainsDependencyInAncestry returns true if the Component or any of
 // its dependencies contains the given path as a dependency.
 func ContainsDependencyInAncestry(c *component.Component, path string) bool {
-	for _, dep := range c.Dependencies {
+	for _, dep := range c.Dependencies() {
 		if dep.Path == path {
 			return true
 		}
@@ -479,8 +498,21 @@ func Parse(
 		parsingCtx = parsingCtx.WithParseOption(parseOptions)
 	}
 
+	var (
+		cfg *config.TerragruntConfig
+		err error
+	)
+
+	// Set a list with partial blocks used to do discovery
+	parsingCtx = parsingCtx.WithDecodeList(
+		config.DependenciesBlock,
+		config.DependencyBlock,
+		config.FeatureFlagsBlock,
+		config.ExcludeBlock,
+	)
+
 	//nolint: contextcheck
-	cfg, err := config.ParseConfigFile(parsingCtx, l, parseOpts.TerragruntConfigPath, nil)
+	cfg, err = config.PartialParseConfigFile(parsingCtx, l, parseOpts.TerragruntConfigPath, nil)
 	if err != nil {
 		if !suppressParseErrors || cfg == nil {
 			l.Debugf("Unrecoverable parse error for %s: %s", parseOpts.TerragruntConfigPath, err)
@@ -493,6 +525,12 @@ func Parse(
 
 	c.Parsed = cfg
 
+	// Populate the Reading field with files read during parsing.
+	// The parsing context tracks all files that were read.
+	if parsingCtx.FilesRead != nil {
+		c.Reading = *parsingCtx.FilesRead
+	}
+
 	return nil
 }
 
@@ -965,7 +1003,7 @@ func (d *Discovery) Discover(
 				l.Debugf("Errors: %w", err)
 			}
 
-			components = dependencyDiscovery.cfgs
+			components = dependencyDiscovery.components
 
 			return nil
 		})
@@ -1017,7 +1055,7 @@ func (d *Discovery) Discover(
 // DependencyDiscovery is the configuration for a DependencyDiscovery.
 type DependencyDiscovery struct {
 	discoveryContext    *component.DiscoveryContext
-	cfgs                component.Components
+	components          component.Components
 	parserOptions       []hclparse.Option
 	depthRemaining      int
 	discoverExternal    bool
@@ -1028,9 +1066,9 @@ type DependencyDiscovery struct {
 // DependencyDiscoveryOption is a function that modifies a DependencyDiscovery.
 type DependencyDiscoveryOption func(*DependencyDiscovery)
 
-func NewDependencyDiscovery(cfgs component.Components, depthRemaining int) *DependencyDiscovery {
+func NewDependencyDiscovery(components component.Components, depthRemaining int) *DependencyDiscovery {
 	return &DependencyDiscovery{
-		cfgs:           cfgs,
+		components:     components,
 		depthRemaining: depthRemaining,
 	}
 }
@@ -1070,7 +1108,7 @@ func (d *DependencyDiscovery) WithDiscoveryContext(discoveryContext *component.D
 func (d *DependencyDiscovery) DiscoverAllDependencies(ctx context.Context, l log.Logger, opts *options.TerragruntOptions) error {
 	errs := []error{}
 
-	for _, cfg := range d.cfgs {
+	for _, cfg := range d.components {
 		if cfg.Kind == component.Stack {
 			continue
 		}
@@ -1088,7 +1126,12 @@ func (d *DependencyDiscovery) DiscoverAllDependencies(ctx context.Context, l log
 	return nil
 }
 
-func (d *DependencyDiscovery) DiscoverDependencies(ctx context.Context, l log.Logger, opts *options.TerragruntOptions, dCfg *component.Component) error {
+func (d *DependencyDiscovery) DiscoverDependencies(
+	ctx context.Context,
+	l log.Logger,
+	opts *options.TerragruntOptions,
+	dComponent *component.Component,
+) error {
 	if d.depthRemaining <= 0 {
 		return errors.New("max dependency depth reached while discovering dependencies")
 	}
@@ -1099,19 +1142,19 @@ func (d *DependencyDiscovery) DiscoverDependencies(ctx context.Context, l log.Lo
 
 	// Stack configs don't have dependencies (at least for now),
 	// so we can return early.
-	if dCfg.Kind == component.Stack {
+	if dComponent.Kind == component.Stack {
 		return nil
 	}
 
 	// This should only happen if we're discovering an ancestor dependency.
-	if dCfg.Parsed == nil {
-		err := Parse(dCfg, ctx, l, opts, d.suppressParseErrors, d.parserOptions)
+	if dComponent.Parsed == nil {
+		err := Parse(dComponent, ctx, l, opts, d.suppressParseErrors, d.parserOptions)
 		if err != nil {
 			return errors.New(err)
 		}
 	}
 
-	dependencyBlocks := dCfg.Parsed.TerragruntDependencies
+	dependencyBlocks := dComponent.Parsed.TerragruntDependencies
 
 	depPaths := make([]string, 0, len(dependencyBlocks))
 
@@ -1127,16 +1170,16 @@ func (d *DependencyDiscovery) DiscoverDependencies(ctx context.Context, l log.Lo
 		depPath := dependency.ConfigPath.AsString()
 
 		if !filepath.IsAbs(depPath) {
-			depPath = filepath.Join(dCfg.Path, depPath)
+			depPath = filepath.Join(dComponent.Path, depPath)
 		}
 
 		depPaths = append(depPaths, depPath)
 	}
 
-	if dCfg.Parsed.Dependencies != nil {
-		for _, dependency := range dCfg.Parsed.Dependencies.Paths {
+	if dComponent.Parsed.Dependencies != nil {
+		for _, dependency := range dComponent.Parsed.Dependencies.Paths {
 			if !filepath.IsAbs(dependency) {
-				dependency = filepath.Join(dCfg.Path, dependency)
+				dependency = filepath.Join(dComponent.Path, dependency)
 			}
 
 			depPaths = append(depPaths, dependency)
@@ -1160,11 +1203,11 @@ func (d *DependencyDiscovery) DiscoverDependencies(ctx context.Context, l log.Lo
 	for depPath := range deduped {
 		external := true
 
-		for _, c := range d.cfgs {
+		for _, c := range d.components {
 			if c.Path == depPath {
 				external = false
 
-				dCfg.Dependencies = append(dCfg.Dependencies, c)
+				dComponent.AddDependency(c)
 
 				continue
 			}
@@ -1186,10 +1229,10 @@ func (d *DependencyDiscovery) DiscoverDependencies(ctx context.Context, l log.Lo
 				ext.DiscoveryContext = d.discoveryContext
 			}
 
-			dCfg.Dependencies = append(dCfg.Dependencies, ext)
+			dComponent.AddDependency(ext)
 
 			if d.discoverExternal {
-				d.cfgs = append(d.cfgs, ext)
+				d.components = append(d.components, ext)
 
 				err := d.DiscoverDependencies(ctx, l, opts, ext)
 				if err != nil {
