@@ -21,7 +21,7 @@ import (
 
 	"github.com/gruntwork-io/terragrunt/config"
 	"github.com/gruntwork-io/terragrunt/config/hclparse"
-	"github.com/gruntwork-io/terragrunt/internal/discovery"
+	"github.com/gruntwork-io/terragrunt/internal/component"
 	"github.com/gruntwork-io/terragrunt/internal/queue"
 	"github.com/gruntwork-io/terragrunt/internal/report"
 	"github.com/gruntwork-io/terragrunt/options"
@@ -43,35 +43,35 @@ func (r *Runner) SetQueue(q *queue.Queue) {
 }
 
 // NewRunnerPoolStack creates a new stack from discovered units.
-func NewRunnerPoolStack(ctx context.Context, l log.Logger, terragruntOptions *options.TerragruntOptions, discovered discovery.DiscoveredConfigs, opts ...common.Option) (common.StackRunner, error) {
+func NewRunnerPoolStack(
+	ctx context.Context,
+	l log.Logger,
+	terragruntOptions *options.TerragruntOptions,
+	discovered component.Components,
+	opts ...common.Option,
+) (common.StackRunner, error) {
 	if len(discovered) == 0 {
-		// If any filtering options are enabled that can result in valid empty results, create an empty runner.
-		isFilteringEnabled := terragruntOptions.StrictInclude ||
-			len(terragruntOptions.ModulesThatInclude) > 0
+		l.Warnf("No units discovered. Creating an empty runner.")
 
-		if isFilteringEnabled {
-			// Create an empty runner that will process no units
-			stack := common.Stack{
-				TerragruntOptions: terragruntOptions,
-				ParserOptions:     config.DefaultParserOptions(l, terragruntOptions),
-			}
-
-			runner := &Runner{
-				Stack: &stack,
-			}
-
-			// Create an empty queue
-			q, queueErr := queue.NewQueue(discovery.DiscoveredConfigs{})
-			if queueErr != nil {
-				return nil, queueErr
-			}
-
-			runner.queue = q
-
-			return runner.WithOptions(opts...), nil
+		// Create an empty runner that will process no units
+		stack := common.Stack{
+			TerragruntOptions: terragruntOptions,
+			ParserOptions:     config.DefaultParserOptions(l, terragruntOptions),
 		}
 
-		return nil, common.ErrNoUnitsFound
+		runner := &Runner{
+			Stack: &stack,
+		}
+
+		// Create an empty queue
+		q, queueErr := queue.NewQueue(component.Components{})
+		if queueErr != nil {
+			return nil, queueErr
+		}
+
+		runner.queue = q
+
+		return runner.WithOptions(opts...), nil
 	}
 
 	// Initialize stack; queue will be constructed after resolving units so we can filter excludes first.
@@ -100,7 +100,7 @@ func NewRunnerPoolStack(ctx context.Context, l log.Logger, terragruntOptions *op
 
 		// Determine per-unit config filename
 		var fname string
-		if cfg.Type == discovery.ConfigTypeStack {
+		if cfg.Kind == component.Stack {
 			fname = config.DefaultStackFile
 		} else {
 			fname = config.DefaultTerragruntConfigPath
@@ -260,14 +260,14 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 		// Build a quick lookup of queue entry status by path to avoid nested scans
 		statusByPath := make(map[string]queue.Status, len(r.queue.Entries))
 		for _, qe := range r.queue.Entries {
-			statusByPath[qe.Config.Path] = qe.Status
+			statusByPath[qe.Component.Path] = qe.Status
 		}
 
 		for _, entry := range r.queue.Entries {
 			if entry.Status == queue.StatusEarlyExit {
-				unit := r.getUnitByPath(entry.Config.Path)
+				unit := r.getUnitByPath(entry.Component.Path)
 				if unit == nil {
-					l.Warnf("Could not find unit for early exit entry: %s", entry.Config.Path)
+					l.Warnf("Could not find unit for early exit entry: %s", entry.Component.Path)
 					continue
 				}
 
@@ -293,7 +293,7 @@ func (r *Runner) Run(ctx context.Context, l log.Logger, opts *options.Terragrunt
 				// If a dependency failed, use it; otherwise if a dependency exited early, use it
 				var failedAncestor string
 
-				for _, dep := range entry.Config.Dependencies {
+				for _, dep := range entry.Component.Dependencies() {
 					status := statusByPath[dep.Path]
 					if status == queue.StatusFailed {
 						failedAncestor = filepath.Base(dep.Path)
@@ -364,7 +364,7 @@ func (r *Runner) getUnitByPath(path string) *common.Unit {
 func (r *Runner) LogUnitDeployOrder(l log.Logger, terraformCommand string) error {
 	outStr := fmt.Sprintf("The runner-pool runner at %s will be processed in the following order for command %s:\n", r.Stack.TerragruntOptions.WorkingDir, terraformCommand)
 	for _, unit := range r.queue.Entries {
-		outStr += fmt.Sprintf("- Unit %s\n", unit.Config.Path)
+		outStr += fmt.Sprintf("- Unit %s\n", unit.Component.Path)
 	}
 
 	l.Info(outStr)
@@ -376,7 +376,7 @@ func (r *Runner) LogUnitDeployOrder(l log.Logger, terraformCommand string) error
 func (r *Runner) JSONUnitDeployOrder(_ string) (string, error) {
 	orderedUnits := make([]string, 0, len(r.queue.Entries))
 	for _, unit := range r.queue.Entries {
-		orderedUnits = append(orderedUnits, unit.Config.Path)
+		orderedUnits = append(orderedUnits, unit.Component.Path)
 	}
 
 	j, err := json.MarshalIndent(orderedUnits, "", "  ")
@@ -392,9 +392,9 @@ func (r *Runner) ListStackDependentUnits() map[string][]string {
 	dependentUnits := make(map[string][]string)
 
 	for _, unit := range r.queue.Entries {
-		if len(unit.Config.Dependencies) != 0 {
-			for _, dep := range unit.Config.Dependencies {
-				dependentUnits[dep.Path] = util.RemoveDuplicatesFromList(append(dependentUnits[dep.Path], unit.Config.Path))
+		if len(unit.Component.Dependencies()) != 0 {
+			for _, dep := range unit.Component.Dependencies() {
+				dependentUnits[dep.Path] = util.RemoveDuplicatesFromList(append(dependentUnits[dep.Path], unit.Component.Path))
 			}
 		}
 	}
@@ -486,7 +486,7 @@ func (r *Runner) summarizePlanAllErrors(l log.Logger, errorStreams []bytes.Buffe
 //   - For each included config, its Dependencies list is filtered to only include included configs.
 //   - The function returns a new slice with shallow-copied entries so the original discovery
 //     results remain unchanged.
-func FilterDiscoveredUnits(discovered discovery.DiscoveredConfigs, units common.Units) discovery.DiscoveredConfigs {
+func FilterDiscoveredUnits(discovered component.Components, units common.Units) component.Components {
 	// Build allowlist from non-excluded unit paths
 	allowed := make(map[string]struct{}, len(units))
 	for _, u := range units {
@@ -496,8 +496,8 @@ func FilterDiscoveredUnits(discovered discovery.DiscoveredConfigs, units common.
 	}
 
 	// First pass: keep only allowed configs and prune their dependencies to allowed ones
-	filtered := make(discovery.DiscoveredConfigs, 0, len(discovered))
-	present := make(map[string]*discovery.DiscoveredConfig, len(discovered))
+	filtered := make(component.Components, 0, len(discovered))
+	present := make(map[string]*component.Component, len(discovered))
 
 	for _, cfg := range discovered {
 		if _, ok := allowed[cfg.Path]; !ok {
@@ -505,19 +505,22 @@ func FilterDiscoveredUnits(discovered discovery.DiscoveredConfigs, units common.
 			continue
 		}
 
-		// Shallow copy unit struct to avoid field drift as values update
-		copyVal := *cfg
-		copyCfg := &copyVal
+		// Create a new component with filtered dependencies
+		copyCfg := &component.Component{
+			Kind:             cfg.Kind,
+			Path:             cfg.Path,
+			DiscoveryContext: cfg.DiscoveryContext,
+			Parsed:           cfg.Parsed,
+			External:         cfg.External,
+			Reading:          cfg.Reading,
+		}
 
-		if cfg.Dependencies != nil {
-			deps := make(discovery.DiscoveredConfigs, 0, len(cfg.Dependencies))
-			for _, dep := range cfg.Dependencies {
+		if len(cfg.Dependencies()) > 0 {
+			for _, dep := range cfg.Dependencies() {
 				if _, ok := allowed[dep.Path]; ok {
-					deps = append(deps, dep)
+					copyCfg.AddDependency(dep)
 				}
 			}
-
-			copyCfg.Dependencies = deps
 		}
 
 		filtered = append(filtered, copyCfg)
@@ -535,8 +538,8 @@ func FilterDiscoveredUnits(discovered discovery.DiscoveredConfigs, units common.
 		}
 
 		// Create a minimal discovered config for the missing unit
-		copyCfg := &discovery.DiscoveredConfig{
-			Type: discovery.ConfigTypeUnit,
+		copyCfg := &component.Component{
+			Kind: component.Unit,
 			Path: u.Path,
 		}
 
@@ -556,8 +559,8 @@ func FilterDiscoveredUnits(discovered discovery.DiscoveredConfigs, units common.
 		}
 
 		// Build a set of existing dependency paths on cfg to avoid duplicates
-		existing := make(map[string]struct{}, len(cfg.Dependencies))
-		for _, dep := range cfg.Dependencies {
+		existing := make(map[string]struct{}, len(cfg.Dependencies()))
+		for _, dep := range cfg.Dependencies() {
 			existing[dep.Path] = struct{}{}
 		}
 
@@ -578,12 +581,12 @@ func FilterDiscoveredUnits(discovered discovery.DiscoveredConfigs, units common.
 			// Ensure the dependency config exists in the filtered set
 			depCfg, ok := present[depUnit.Path]
 			if !ok {
-				depCfg = &discovery.DiscoveredConfig{Type: discovery.ConfigTypeUnit, Path: depUnit.Path}
+				depCfg = &component.Component{Kind: component.Unit, Path: depUnit.Path}
 				filtered = append(filtered, depCfg)
 				present[depUnit.Path] = depCfg
 			}
 
-			cfg.Dependencies = append(cfg.Dependencies, depCfg)
+			cfg.AddDependency(depCfg)
 		}
 	}
 
